#include <hardware.h>
#include "font_pearl_8x8.c"
#include "font_5x7.c"
#include "boot.xbm"

#include <stdarg.h>
#include <stdio.h>

// lcd is 96 x 64 pixels
// 19 x 8 chars
#define LCD_W 96
#define LCD_H 64

#define CHAR_W 5

// the A0 line is used to determine which mode the byte sent is to be interpreted in
#define LCD_SET_COMMAND_MODE     PORT5.DR.BIT.B1 = 0
#define LCD_SET_DATA_MODE        PORT5.DR.BIT.B1 = 1

static void lcd_out_byte(uint8_t byte, uint8_t isCommand)
{
    if (!spi_select(SPI_DEVICE_LCD))
	return;
    if (isCommand) LCD_SET_COMMAND_MODE;
    else LCD_SET_DATA_MODE;
    spi_write(&byte, 1);
    spi_release();
}


static uint8_t lcd_in_byte (uint8_t isCommand)
{
  uint8_t byte;
    if (!spi_select(SPI_DEVICE_LCD))
	    return 0;
    if (isCommand) LCD_SET_COMMAND_MODE;
      else LCD_SET_DATA_MODE;
    spi_read (&byte, 1);
    spi_release();
    
  return byte;
}



static void lcd_command(int8_t command)
{
    lcd_out_byte(command, 1);
}

static void lcd_write(int8_t data)
{
    lcd_out_byte(data, 0);
}

uint8_t lcd_read (void)
{
  return lcd_in_byte (0);
}

static void lcd_set_x(uint8_t xx)
{
    lcd_command(LCD_COMMAND_PAGE_ZERO);
    lcd_command(0x80 | xx);
}

static void lcd_set_y_page(uint8_t yy)
{
    lcd_command(LCD_COMMAND_PAGE_ZERO);
    lcd_command(0x40 | yy);	
}

static void lcd_set_address(uint8_t yy, uint8_t xx)
{
    lcd_set_y_page(yy);
    lcd_set_x(xx);
}

void lcd_open()
{
    int i = 0 ;
    
    PORTC.DDR.BIT.B3 = 1 ;   // LCD GPIO for Reset LCD
    PORT5.DDR.BIT.B1 = 1 ;   // LCD RS A0

    PORTC.DR.BIT.B3 = 1 ;    // Reset LCD
    for (i=0;i<1000000;i++)asm("");
    PORTC.DR.BIT.B3 = 0 ;    // clear LCD reset line
    for (i=0;i<1000000;i++)asm("");

    // set up the bias and boost
    lcd_command(LCD_COMMAND_PAGE_ONE);
    lcd_command(0x16); // bias
    lcd_command(LCD_COMMAND_PAGE_THREE);
    lcd_command(0x9B); // boost

    // set VO range
    lcd_command(LCD_COMMAND_PAGE_ZERO);
    lcd_command(0x4);
    lcd_command(LCD_COMMAND_PAGE_ONE);
    lcd_command(0x80 | 96);

    // clear the ram
    lcd_set_x(0);
    lcd_set_y_page(0);
    
    for (i =0 ; i < 808; i++)
	    lcd_write(0x00);

    lcd_command(LCD_COMMAND_PAGE_ZERO);
    lcd_command(0x8 | 0x4);
    
    lcd_boot_logo ();

}

unsigned char screenbuf[96][8];

void
lcd_pset (uint8_t x, uint8_t y)
{
  unsigned char byte;
  
  byte = (screenbuf[x][y >> 3] |= (1 << (y & 7)));
  
  lcd_set_address (y >> 3, x); 
  lcd_write (byte);
}


void
lcd_byte (uint8_t x, uint8_t y, uint8_t byte)
{
  screenbuf[x][y >> 3] = byte;
  
  lcd_set_address (y >> 3, x); 
  lcd_write (byte);
}


void
lcd_preset (uint8_t x, uint8_t y)
{
  unsigned char byte;
  
  byte = (screenbuf[x][y >> 3] &= ~(1 << (y & 7)));
  
  lcd_set_address (y >> 3, x); 
  lcd_write (byte);
}

int curr_row, curr_col;
int cgrom_col, cgrom_row;


void
lcd_clear_chars (void)
{
  int i, j;
  
  for (j = 0; j < 8; j++)
    for (i = 0; i < 96; i++)
      lcd_byte (i, j << 3, 0);
      
  curr_row = 0;
  curr_col = 0;
  cgrom_row = 0;
  cgrom_col = 0;
}

void
lcd_gotoxy (int x, int y)
{
  curr_col = x;
  curr_row = y;
}

void
lcd_cgrom_gotoxy (int x, int y)
{
  cgrom_col = x;
  cgrom_row = y;
}


void
lcd_display_char (char c)
{
  int i, j;
  char byte;
  
  for (i = 0; i < 8; i++)
  {
    byte = 0;
    
    for (j = 0; j < 8; j++)
      byte |= !! (fontdata_pearl8x8 [(c << 3) + j] & (1 << (7 - i))) << j;
    
    lcd_byte ((curr_col << 3) + i, curr_row << 3, byte); 
  }
  

  curr_col++;
  if (curr_col >= 12)
  {
    curr_col = 0;
    curr_row++;
  }
  
  if (curr_row >= 8)
    curr_row = 7;
}

void
lcd_boot_logo (void)
{
  int x, y;
  
  for (x = 0; x < 96; x++)
    for (y = 0; y < 8; y++)
    {
      lcd_set_address (y, x); 
      lcd_write (boot_bits[x * 8 + y]);
    }
}

void
lcd_cgrom_display_char (char c)
{
  int i, j;
  char byte;
  
  for (i = 0; i < 5; i++)
  {
    byte = 0;
    
    for (j = 0; j < 7; j++)
      byte |= !! (font_5x7 [c * 7 + j] & (1 << i)) << j;
    
    lcd_byte (cgrom_col * 5 + i, cgrom_row * 7, byte); 
  }
  

  cgrom_col++;
  
  if (cgrom_col >= 19)
  {
    cgrom_col = 0;
    cgrom_row++;
  }
  
  if (cgrom_row >= 9)
    cgrom_row = 8;
}


void
lcd_string (const char *str)
{
  int i;
  
  while (*str)
    lcd_display_char (*str++);
}


void
lcd_cgrom_string (const char *str)
{
  int i;
  
  while (*str)
    lcd_cgrom_display_char (*str++);
}
 
void 
lcd_printf (const char *fmt, ...)
{
  char message[8 * 12];
  va_list ap;
  va_start (ap, fmt);
  int len = vsnprintf (message, sizeof(message) - 1, fmt, ap);
  va_end (ap);

    
  lcd_string (message);      
}

void 
lcd_cgrom_printf (const char *fmt, ...)
{
  char message [9 * 19];
  va_list ap;
  va_start (ap, fmt);
  int len = vsnprintf (message, sizeof (message) - 1, fmt, ap);
  va_end (ap);

    
  lcd_cgrom_string (message);
}

